@@ -77,7 +77,10 @@
 #define GTK_TEXT_USE_INTERNAL_UNSUPPORTED_API
 #include "config.h"
 #include "gtktextdisplay.h"
+#include "gtkwidgetprivate.h"
+#include "gtkstylecontextprivate.h"
 #include "gtkintl.h"
+
 /* DO NOT go putting private headers in here. This file should only
  * use the semi-public headers, as with gtktextview.c.
  */
@@ -118,22 +121,6 @@ struct _GtkTextRendererClass
 
 G_DEFINE_TYPE (GtkTextRenderer, _gtk_text_renderer, PANGO_TYPE_RENDERER)
 
-static GdkColor *
-text_renderer_get_error_color (GtkTextRenderer *text_renderer)
-{
-  static const GdkColor red = { 0, 0xffff, 0, 0 };
-
-  if (!text_renderer->error_color)
-    gtk_widget_style_get (text_renderer->widget,
-			  "error-underline-color", &text_renderer->error_color,
-			  NULL);
-  
-  if (!text_renderer->error_color)
-    text_renderer->error_color = gdk_color_copy (&red);
-
-  return text_renderer->error_color;
-}
-
 static void
 text_renderer_set_gdk_color (GtkTextRenderer *text_renderer,
 			     PangoRenderPart  part,
@@ -153,7 +140,27 @@ text_renderer_set_gdk_color (GtkTextRenderer *text_renderer,
     }
   else
     pango_renderer_set_color (renderer, part, NULL);
-	   
+}
+
+static void
+text_renderer_set_rgba (GtkTextRenderer *text_renderer,
+			PangoRenderPart  part,
+			GdkRGBA         *rgba)
+{
+  PangoRenderer *renderer = PANGO_RENDERER (text_renderer);
+
+  if (rgba)
+    {
+      PangoColor color;
+
+      color.red = CLAMP (rgba->red * 65535. + 0.5, 0, 65535);
+      color.green = CLAMP (rgba->green * 65535. + 0.5, 0, 65535);
+      color.blue = CLAMP (rgba->blue * 65535. + 0.5, 0, 65535);
+
+      pango_renderer_set_color (renderer, part, &color);
+    }
+  else
+    pango_renderer_set_color (renderer, part, NULL);
 }
 
 static GtkTextAppearance *
@@ -178,8 +185,12 @@ static void
 gtk_text_renderer_prepare_run (PangoRenderer  *renderer,
 			       PangoLayoutRun *run)
 {
+  GtkStyleContext *context;
+  GtkStateFlags state;
   GtkTextRenderer *text_renderer = GTK_TEXT_RENDERER (renderer);
-  GdkColor *bg_color, *fg_color, *underline_color;
+  GdkColor *bg_color = NULL;
+  GdkRGBA *fg_rgba = NULL;
+  GdkColor *fg_color = NULL;
   GtkTextAppearance *appearance;
 
   PANGO_RENDERER_CLASS (_gtk_text_renderer_parent_class)->prepare_run (renderer, run);
@@ -187,6 +198,10 @@ gtk_text_renderer_prepare_run (PangoRenderer  *renderer,
   appearance = get_item_appearance (run->item);
   g_assert (appearance != NULL);
 
+  context = gtk_widget_get_style_context (text_renderer->widget);
+
+  state = gtk_widget_get_state_flags (text_renderer->widget);
+
   if (appearance->draw_bg && text_renderer->state == NORMAL)
     bg_color = &appearance->bg_color;
   else
@@ -196,25 +211,53 @@ gtk_text_renderer_prepare_run (PangoRenderer  *renderer,
 
   if (text_renderer->state == SELECTED)
     {
-      if (gtk_widget_has_focus (text_renderer->widget))
-	fg_color = &text_renderer->widget->style->text[GTK_STATE_SELECTED];
-      else
-	fg_color = &text_renderer->widget->style->text[GTK_STATE_ACTIVE];
+      state |= GTK_STATE_FLAG_SELECTED;
+
+      gtk_style_context_get (context, state,
+                             "color", &fg_rgba,
+                              NULL);
     }
   else if (text_renderer->state == CURSOR && gtk_widget_has_focus (text_renderer->widget))
-    fg_color = &text_renderer->widget->style->base[GTK_STATE_NORMAL];
+    {
+      gtk_style_context_get (context, state,
+                             "background-color", &fg_rgba,
+                              NULL);
+    }
   else
     fg_color = &appearance->fg_color;
 
-  text_renderer_set_gdk_color (text_renderer, PANGO_RENDER_PART_FOREGROUND, fg_color);
-  text_renderer_set_gdk_color (text_renderer, PANGO_RENDER_PART_STRIKETHROUGH, fg_color);
+  if (fg_rgba)
+    {
+      text_renderer_set_rgba (text_renderer, PANGO_RENDER_PART_FOREGROUND, fg_rgba);
+      text_renderer_set_rgba (text_renderer, PANGO_RENDER_PART_STRIKETHROUGH,fg_rgba);
+    }
+  else
+    {
+      text_renderer_set_gdk_color (text_renderer, PANGO_RENDER_PART_FOREGROUND, fg_color);
+      text_renderer_set_gdk_color (text_renderer, PANGO_RENDER_PART_STRIKETHROUGH, fg_color);
+    }
 
   if (appearance->underline == PANGO_UNDERLINE_ERROR)
-    underline_color = text_renderer_get_error_color (text_renderer);
-  else
-    underline_color = fg_color;
+    {
+      if (!text_renderer->error_color)
+        {
+          gtk_style_context_get_style (context,
+                                       "error-underline-color", &text_renderer->error_color,
+                                       NULL);
 
-  text_renderer_set_gdk_color (text_renderer, PANGO_RENDER_PART_UNDERLINE, underline_color);
+          if (!text_renderer->error_color)
+            {
+              static const GdkColor red = { 0, 0xffff, 0, 0 };
+              text_renderer->error_color = gdk_color_copy (&red);
+            }
+        }
+
+      text_renderer_set_gdk_color (text_renderer, PANGO_RENDER_PART_UNDERLINE, text_renderer->error_color);
+    }
+  else if (fg_rgba)
+    text_renderer_set_rgba (text_renderer, PANGO_RENDER_PART_UNDERLINE, fg_rgba);
+  else
+    text_renderer_set_gdk_color (text_renderer, PANGO_RENDER_PART_UNDERLINE, fg_color);
 }
 
 static void
@@ -352,20 +395,7 @@ gtk_text_renderer_draw_shape (PangoRenderer   *renderer,
 			      int              y)
 {
   GtkTextRenderer *text_renderer = GTK_TEXT_RENDERER (renderer);
-  GdkColor *fg;
 
-  if (text_renderer->state == SELECTED)
-    {
-      if (gtk_widget_has_focus (text_renderer->widget))
-	fg = &text_renderer->widget->style->text[GTK_STATE_SELECTED];
-      else
-	fg = &text_renderer->widget->style->text[GTK_STATE_SELECTED];
-    }
-  else if (text_renderer->state == CURSOR && gtk_widget_has_focus (text_renderer->widget))
-    fg = &text_renderer->widget->style->base[GTK_STATE_NORMAL];
-  else
-    fg = &text_renderer->widget->style->text[GTK_STATE_NORMAL];
-  
   if (attr->data == NULL)
     {
       /* This happens if we have an empty widget anchor. Draw
@@ -373,18 +403,17 @@ gtk_text_renderer_draw_shape (PangoRenderer   *renderer,
        */
       GdkRectangle shape_rect;
       cairo_t *cr;
-      
+
       shape_rect.x = PANGO_PIXELS (x);
       shape_rect.y = PANGO_PIXELS (y + attr->logical_rect.y);
       shape_rect.width = PANGO_PIXELS (x + attr->logical_rect.width) - shape_rect.x;
       shape_rect.height = PANGO_PIXELS (y + attr->logical_rect.y + attr->logical_rect.height) - shape_rect.y;
-      
+
+      set_color (text_renderer, PANGO_RENDER_PART_FOREGROUND);
+
       cr = text_renderer->cr;
 
-      cairo_save (cr);
-
       cairo_set_line_width (cr, 1.0);
-      gdk_cairo_set_source_color (cr, fg);
 
       cairo_rectangle (cr,
                        shape_rect.x + 0.5, shape_rect.y + 0.5,
@@ -399,8 +428,8 @@ gtk_text_renderer_draw_shape (PangoRenderer   *renderer,
                      shape_rect.y + 0.5);
 
       cairo_stroke (cr);
-      
-      cairo_restore (cr);
+
+      unset_color (text_renderer);
     }
   else if (GDK_IS_PIXBUF (attr->data))
     {
@@ -470,8 +499,24 @@ text_renderer_begin (GtkTextRenderer *text_renderer,
                      GtkWidget       *widget,
                      cairo_t         *cr)
 {
+  GtkStyleContext *context;
+  GtkStateFlags state;
+  GdkRGBA color;
+
   text_renderer->widget = widget;
   text_renderer->cr = cr;
+
+  context = gtk_widget_get_style_context (widget);
+
+  gtk_style_context_save (context);
+  gtk_style_context_add_class (context, GTK_STYLE_CLASS_VIEW);
+
+  state = gtk_widget_get_state_flags (widget);
+  gtk_style_context_get_color (context, state, &color);
+
+  cairo_save (cr);
+
+  gdk_cairo_set_source_rgba (cr, &color);
 }
 
 /* Returns a GSList of (referenced) widgets encountered while drawing.
@@ -479,8 +524,15 @@ text_renderer_begin (GtkTextRenderer *text_renderer,
 static GList *
 text_renderer_end (GtkTextRenderer *text_renderer)
 {
+  GtkStyleContext *context;
   GList *widgets = text_renderer->widgets;
 
+  cairo_restore (text_renderer->cr);
+
+  context = gtk_widget_get_style_context (text_renderer->widget);
+
+  gtk_style_context_restore (context);
+
   text_renderer->widget = NULL;
   text_renderer->cr = NULL;
 
@@ -495,7 +547,6 @@ text_renderer_end (GtkTextRenderer *text_renderer)
   return widgets;
 }
 
-
 static cairo_region_t *
 get_selected_clip (GtkTextRenderer    *text_renderer,
                    PangoLayout        *layout,
@@ -531,20 +582,17 @@ get_selected_clip (GtkTextRenderer    *text_renderer,
 static void
 render_para (GtkTextRenderer    *text_renderer,
              GtkTextLineDisplay *line_display,
-             /* Top-left corner of paragraph including all margins */
-             int                 x,
-             int                 y,
              int                 selection_start_index,
              int                 selection_end_index)
 {
+  GtkStyleContext *context;
+  GtkStateFlags state;
   PangoLayout *layout = line_display->layout;
   int byte_offset = 0;
   PangoLayoutIter *iter;
   PangoRectangle layout_logical;
   int screen_width;
-  GdkColor *selection;
-  gint state;
-  
+  GdkRGBA selection;
   gboolean first = TRUE;
 
   iter = pango_layout_get_iter (layout);
@@ -557,13 +605,14 @@ render_para (GtkTextRenderer    *text_renderer,
   layout_logical.y += line_display->top_margin * PANGO_SCALE;
 
   screen_width = line_display->total_width;
-  
-  if (gtk_widget_has_focus (text_renderer->widget))
-    state = GTK_STATE_SELECTED;
-  else
-    state = GTK_STATE_ACTIVE;
 
-  selection = &text_renderer->widget->style->base [state];
+  context = gtk_widget_get_style_context (text_renderer->widget);
+
+  state = GTK_STATE_FLAG_SELECTED;
+  if (gtk_widget_has_focus (text_renderer->widget))
+    state |= GTK_STATE_FLAG_FOCUSED;
+
+  gtk_style_context_get_background_color (context, state, &selection);
 
   do
     {
@@ -587,7 +636,7 @@ render_para (GtkTextRenderer    *text_renderer,
       /* Selection is the height of the line, plus top/bottom
        * margin if we're the first/last line
        */
-      selection_y = y + PANGO_PIXELS (first_y) + line_display->top_margin;
+      selection_y = PANGO_PIXELS (first_y) + line_display->top_margin;
       selection_height = PANGO_PIXELS (last_y) - PANGO_PIXELS (first_y);
 
       if (first)
@@ -608,9 +657,9 @@ render_para (GtkTextRenderer    *text_renderer,
           cairo_t *cr = text_renderer->cr;
 
           cairo_save (cr);
-          gdk_cairo_set_source_color (cr, selection);
+          gdk_cairo_set_source_rgba (cr, &selection);
           cairo_rectangle (cr, 
-                           x + line_display->left_margin, selection_y,
+                           line_display->left_margin, selection_y,
                            screen_width, selection_height);
           cairo_fill (cr);
           cairo_restore(cr);
@@ -618,8 +667,8 @@ render_para (GtkTextRenderer    *text_renderer,
 	  text_renderer_set_state (text_renderer, SELECTED);
 	  pango_renderer_draw_layout_line (PANGO_RENDERER (text_renderer),
 					   line, 
-					   PANGO_SCALE * x + line_rect.x,
-					   PANGO_SCALE * y + baseline);
+					   line_rect.x,
+					   baseline);
         }
       else
         {
@@ -631,7 +680,7 @@ render_para (GtkTextRenderer    *text_renderer,
 
               gdk_cairo_set_source_color (cr, line_display->pg_bg_color);
               cairo_rectangle (cr, 
-                               x + line_display->left_margin, selection_y,
+                               line_display->left_margin, selection_y,
                                screen_width, selection_height);
               cairo_fill (cr);
 
@@ -641,8 +690,8 @@ render_para (GtkTextRenderer    *text_renderer,
 	  text_renderer_set_state (text_renderer, NORMAL);
 	  pango_renderer_draw_layout_line (PANGO_RENDERER (text_renderer),
 					   line, 
-					   PANGO_SCALE * x + line_rect.x,
-					   PANGO_SCALE * y + baseline);
+					   line_rect.x,
+					   baseline);
 
 	  /* Check if some part of the line is selected; the newline
 	   * that is after line->length for the last line of the
@@ -654,7 +703,7 @@ render_para (GtkTextRenderer    *text_renderer,
             {
               cairo_t *cr = text_renderer->cr;
               cairo_region_t *clip_region = get_selected_clip (text_renderer, layout, line,
-                                                          x + line_display->x_offset,
+                                                          line_display->x_offset,
                                                           selection_y,
                                                           selection_height,
                                                           selection_start_index, selection_end_index);
@@ -664,9 +713,9 @@ render_para (GtkTextRenderer    *text_renderer,
               cairo_clip (cr);
               cairo_region_destroy (clip_region);
 
-              gdk_cairo_set_source_color (cr, selection);
+              gdk_cairo_set_source_rgba (cr, &selection);
               cairo_rectangle (cr,
-                               x + PANGO_PIXELS (line_rect.x),
+                               PANGO_PIXELS (line_rect.x),
                                selection_y,
                                PANGO_PIXELS (line_rect.width),
                                selection_height);
@@ -675,8 +724,8 @@ render_para (GtkTextRenderer    *text_renderer,
 	      text_renderer_set_state (text_renderer, SELECTED);
 	      pango_renderer_draw_layout_line (PANGO_RENDERER (text_renderer),
 					       line, 
-					       PANGO_SCALE * x + line_rect.x,
-					       PANGO_SCALE * y + baseline);
+					       line_rect.x,
+					       baseline);
 
               cairo_restore (cr);
 
@@ -687,9 +736,9 @@ render_para (GtkTextRenderer    *text_renderer,
                 {
                   cairo_save (cr);
 
-                  gdk_cairo_set_source_color (cr, selection);
+                  gdk_cairo_set_source_rgba (cr, &selection);
                   cairo_rectangle (cr,
-                                   x + line_display->left_margin,
+                                   line_display->left_margin,
                                    selection_y,
                                    PANGO_PIXELS (line_rect.x) - line_display->left_margin,
                                    selection_height);
@@ -711,9 +760,9 @@ render_para (GtkTextRenderer    *text_renderer,
 
                   cairo_save (cr);
 
-                  gdk_cairo_set_source_color (cr, selection);
+                  gdk_cairo_set_source_rgba (cr, &selection);
                   cairo_rectangle (cr,
-                                   x + PANGO_PIXELS (line_rect.x) + PANGO_PIXELS (line_rect.width),
+                                   PANGO_PIXELS (line_rect.x) + PANGO_PIXELS (line_rect.width),
                                    selection_y,
                                    nonlayout_width,
                                    selection_height);
@@ -729,15 +778,15 @@ render_para (GtkTextRenderer    *text_renderer,
 		    (at_last_line && line_display->insert_index == byte_offset + line->length)))
 	    {
 	      GdkRectangle cursor_rect;
-              GdkColor cursor_color;
+              GdkRGBA cursor_color;
               cairo_t *cr = text_renderer->cr;
 
-	      /* we draw text using base color on filled cursor rectangle of cursor color
-	       * (normally white on black) */
-	      _gtk_widget_get_cursor_color (text_renderer->widget, &cursor_color);
+              /* we draw text using base color on filled cursor rectangle of cursor color
+               * (normally white on black) */
+              _gtk_style_context_get_cursor_color (context, &cursor_color, NULL);
 
-	      cursor_rect.x = x + line_display->x_offset + line_display->block_cursor.x;
-	      cursor_rect.y = y + line_display->block_cursor.y + line_display->top_margin;
+	      cursor_rect.x = line_display->x_offset + line_display->block_cursor.x;
+	      cursor_rect.y = line_display->block_cursor.y + line_display->top_margin;
 	      cursor_rect.width = line_display->block_cursor.width;
 	      cursor_rect.height = line_display->block_cursor.height;
 
@@ -746,23 +795,26 @@ render_para (GtkTextRenderer    *text_renderer,
               gdk_cairo_rectangle (cr, &cursor_rect);
               cairo_clip (cr);
 
-              gdk_cairo_set_source_color (cr, &cursor_color);
+              gdk_cairo_set_source_rgba (cr, &cursor_color);
               cairo_paint (cr);
 
-	      /* draw text under the cursor if any */
-	      if (!line_display->cursor_at_line_end)
-		{
-                  gdk_cairo_set_source_color (cr,
-                                              &text_renderer->widget->style->base[text_renderer->widget->state]);
+              /* draw text under the cursor if any */
+              if (!line_display->cursor_at_line_end)
+                {
+                  GdkRGBA color;
+
+                  state = gtk_widget_get_state_flags (text_renderer->widget);
+                  gtk_style_context_get_background_color (context, state, &color);
+
+                  gdk_cairo_set_source_rgba (cr, &color);
 
 		  text_renderer_set_state (text_renderer, CURSOR);
 
 		  pango_renderer_draw_layout_line (PANGO_RENDERER (text_renderer),
 						   line,
-						   PANGO_SCALE * x + line_rect.x,
-						   PANGO_SCALE * y + baseline);
-
-		}
+						   line_rect.x,
+						   baseline);
+                }
 
               cairo_restore (cr);
 	    }
@@ -789,23 +841,10 @@ get_text_renderer (void)
 void
 gtk_text_layout_draw (GtkTextLayout *layout,
                       GtkWidget *widget,
-                      GdkDrawable *drawable,
-		      gpointer     cursor_gc,
-                      /* Location of the drawable
-                         in layout coordinates */
-                      gint x_offset,
-                      gint y_offset,
-                      /* Region of the layout to
-                         render */
-                      gint x,
-                      gint y,
-                      gint width,
-                      gint height,
-                      /* widgets to expose */
+                      cairo_t *cr,
                       GList **widgets)
 {
-  GdkRectangle clip;
-  gint current_y;
+  gint offset_y;
   GSList *cursor_list;
   GtkTextRenderer *text_renderer;
   GtkTextIter selection_start, selection_end;
@@ -813,33 +852,27 @@ gtk_text_layout_draw (GtkTextLayout *layout,
   GSList *line_list;
   GSList *tmp_list;
   GList *tmp_widgets;
-  cairo_t *cr;
-  
+  GdkRectangle clip;
+
   g_return_if_fail (GTK_IS_TEXT_LAYOUT (layout));
   g_return_if_fail (layout->default_style != NULL);
   g_return_if_fail (layout->buffer != NULL);
-  g_return_if_fail (drawable != NULL);
-  g_return_if_fail (width >= 0);
-  g_return_if_fail (height >= 0);
+  g_return_if_fail (cr != NULL);
 
-  if (width == 0 || height == 0)
+  if (!gdk_cairo_get_clip_rectangle (cr, &clip))
     return;
 
-  line_list =  gtk_text_layout_get_lines (layout, y + y_offset, y + y_offset + height, &current_y);
-  current_y -= y_offset;
+  line_list = gtk_text_layout_get_lines (layout, clip.y, clip.y + clip.height, &offset_y);
 
   if (line_list == NULL)
     return; /* nothing on the screen */
 
-  cr = gdk_cairo_create (drawable);
-  cairo_rectangle (cr, x, y, width, height);
-  cairo_clip (cr);
-
-  gdk_cairo_set_source_color (cr, &widget->style->text[widget->state]);
-
   text_renderer = get_text_renderer ();
   text_renderer_begin (text_renderer, widget, cr);
 
+  /* text_renderer_begin/end does cairo_save/restore */
+  cairo_translate (cr, 0, offset_y);
+
   gtk_text_layout_wrap_loop_start (layout);
 
   if (gtk_text_buffer_get_selection_bounds (layout->buffer,
@@ -893,8 +926,6 @@ gtk_text_layout_draw (GtkTextLayout *layout,
             }
 
           render_para (text_renderer, line_display,
-                       - x_offset,
-                       current_y,
                        selection_start_index, selection_end_index);
 
           /* We paint the cursors last, because they overlap another chunk
@@ -929,20 +960,20 @@ gtk_text_layout_draw (GtkTextLayout *layout,
  		    dir = (dir == GTK_TEXT_DIR_RTL) ? GTK_TEXT_DIR_LTR : GTK_TEXT_DIR_RTL;
  		}
  
- 	      cursor_location.x = line_display->x_offset + cursor->x - x_offset;
- 	      cursor_location.y = current_y + line_display->top_margin + cursor->y;
+ 	      cursor_location.x = line_display->x_offset + cursor->x;
+ 	      cursor_location.y = line_display->top_margin + cursor->y;
  	      cursor_location.width = 0;
  	      cursor_location.height = cursor->height;
 
-	      gtk_draw_insertion_cursor (widget, drawable, &clip, &cursor_location,
-					 cursor->is_strong,
-					 dir, have_strong && have_weak);
+	      gtk_draw_insertion_cursor (widget, cr, &cursor_location,
+                                         cursor->is_strong,
+                                         dir, have_strong && have_weak);
 
               cursor_list = cursor_list->next;
             }
         } /* line_display->height > 0 */
           
-      current_y += line_display->height;
+      cairo_translate (cr, 0, line_display->height);
       gtk_text_layout_free_line_display (layout, line_display);
       
       tmp_list = g_slist_next (tmp_list);
@@ -959,7 +990,5 @@ gtk_text_layout_draw (GtkTextLayout *layout,
       g_list_free (tmp_widgets);
     }
 
-  cairo_destroy (cr);
-
   g_slist_free (line_list);
 }
