@@ -1,4 +1,4 @@
-/* gtktoggletoolbutton.c
+ /* gtktoggletoolbutton.c
  *
  * Copyright (C) 2002 Anders Carlsson <andersca@gnome.org>
  * Copyright (C) 2002 James Henstridge <james@daa.com.au>
@@ -27,8 +27,10 @@
 #include "gtkstock.h"
 #include "gtkintl.h"
 #include "gtkradiotoolbutton.h"
+#include "gtktoggleaction.h"
+#include "gtkactivatable.h"
 #include "gtkprivate.h"
-#include "gtkalias.h"
+
 
 #define MENU_ID "gtk-toggle-tool-button-menu-id"
 
@@ -43,8 +45,6 @@ enum {
 };
 
 
-#define GTK_TOGGLE_TOOL_BUTTON_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_TOGGLE_TOOL_BUTTON, GtkToggleToolButtonPrivate))
-
 struct _GtkToggleToolButtonPrivate
 {
   guint active : 1;
@@ -67,9 +67,20 @@ static void button_toggled      (GtkWidget           *widget,
 static void menu_item_activated (GtkWidget           *widget,
 				 GtkToggleToolButton *button);
 
-static guint         toggle_signals[LAST_SIGNAL] = { 0 };
 
-G_DEFINE_TYPE (GtkToggleToolButton, gtk_toggle_tool_button, GTK_TYPE_TOOL_BUTTON)
+static void gtk_toggle_tool_button_activatable_interface_init (GtkActivatableIface  *iface);
+static void gtk_toggle_tool_button_update                     (GtkActivatable       *activatable,
+							       GtkAction            *action,
+							       const gchar          *property_name);
+static void gtk_toggle_tool_button_sync_action_properties     (GtkActivatable       *activatable,
+							       GtkAction            *action);
+
+static GtkActivatableIface *parent_activatable_iface;
+static guint                toggle_signals[LAST_SIGNAL] = { 0 };
+
+G_DEFINE_TYPE_WITH_CODE (GtkToggleToolButton, gtk_toggle_tool_button, GTK_TYPE_TOOL_BUTTON,
+			 G_IMPLEMENT_INTERFACE (GTK_TYPE_ACTIVATABLE,
+						gtk_toggle_tool_button_activatable_interface_init))
 
 static void
 gtk_toggle_tool_button_class_init (GtkToggleToolButtonClass *klass)
@@ -91,7 +102,7 @@ gtk_toggle_tool_button_class_init (GtkToggleToolButtonClass *klass)
   /**
    * GtkToggleToolButton:active:
    *
-   * If the toggle tool button should be pressed in or not.
+   * If the toggle tool button should be pressed in.
    *
    * Since: 2.8
    */
@@ -99,7 +110,7 @@ gtk_toggle_tool_button_class_init (GtkToggleToolButtonClass *klass)
                                    PROP_ACTIVE,
                                    g_param_spec_boolean ("active",
 							 P_("Active"),
-							 P_("If the toggle button should be pressed in or not"),
+							 P_("If the toggle button should be pressed in"),
 							 FALSE,
 							 GTK_PARAM_READWRITE));
 
@@ -127,7 +138,9 @@ gtk_toggle_tool_button_init (GtkToggleToolButton *button)
   GtkToolButton *tool_button = GTK_TOOL_BUTTON (button);
   GtkToggleButton *toggle_button = GTK_TOGGLE_BUTTON (_gtk_tool_button_get_button (tool_button));
 
-  button->priv = GTK_TOGGLE_TOOL_BUTTON_GET_PRIVATE (button);
+  button->priv = G_TYPE_INSTANCE_GET_PRIVATE (button,
+                                              GTK_TYPE_TOGGLE_TOOL_BUTTON,
+                                              GtkToggleToolButtonPrivate);
 
   /* If the real button is a radio button, it may have been
    * active at the time it was created.
@@ -188,10 +201,16 @@ gtk_toggle_tool_button_create_menu_proxy (GtkToolItem *item)
   GtkStockItem stock_item;
   gboolean use_mnemonic = TRUE;
   const char *label;
+  GtkWidget *label_widget;
+  const gchar *label_text;
+  const gchar *stock_id;
 
-  GtkWidget *label_widget = gtk_tool_button_get_label_widget (tool_button);
-  const gchar *label_text = gtk_tool_button_get_label (tool_button);
-  const gchar *stock_id = gtk_tool_button_get_stock_id (tool_button);
+  if (_gtk_tool_item_create_menu_proxy (item))
+    return TRUE;
+
+  label_widget = gtk_tool_button_get_label_widget (tool_button);
+  label_text = gtk_tool_button_get_label (tool_button);
+  stock_id = gtk_tool_button_get_stock_id (tool_button);
 
   if (GTK_IS_LABEL (label_widget))
     {
@@ -272,7 +291,9 @@ static void
 button_toggled (GtkWidget           *widget,
 		GtkToggleToolButton *toggle_tool_button)
 {
-  gboolean toggle_active = GTK_TOGGLE_BUTTON (widget)->active;
+  gboolean toggle_active;
+
+  toggle_active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 
   if (toggle_tool_button->priv->active != toggle_active)
     {
@@ -292,6 +313,52 @@ button_toggled (GtkWidget           *widget,
     }
 }
 
+static void
+gtk_toggle_tool_button_activatable_interface_init (GtkActivatableIface *iface)
+{
+  parent_activatable_iface = g_type_interface_peek_parent (iface);
+  iface->update = gtk_toggle_tool_button_update;
+  iface->sync_action_properties = gtk_toggle_tool_button_sync_action_properties;
+}
+
+static void
+gtk_toggle_tool_button_update (GtkActivatable *activatable,
+			       GtkAction      *action,
+			       const gchar    *property_name)
+{
+  GtkToggleToolButton *button;
+
+  parent_activatable_iface->update (activatable, action, property_name);
+
+  button = GTK_TOGGLE_TOOL_BUTTON (activatable);
+
+  if (strcmp (property_name, "active") == 0)
+    {
+      gtk_action_block_activate (action);
+      gtk_toggle_tool_button_set_active (button, gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
+      gtk_action_unblock_activate (action);
+    }
+}
+
+static void
+gtk_toggle_tool_button_sync_action_properties (GtkActivatable *activatable,
+					       GtkAction      *action)
+{
+  GtkToggleToolButton *button;
+
+  parent_activatable_iface->sync_action_properties (activatable, action);
+
+  if (!GTK_IS_TOGGLE_ACTION (action))
+    return;
+
+  button = GTK_TOGGLE_TOOL_BUTTON (activatable);
+
+  gtk_action_block_activate (action);
+  gtk_toggle_tool_button_set_active (button, gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
+  gtk_action_unblock_activate (action);
+}
+
+
 /**
  * gtk_toggle_tool_button_new:
  * 
@@ -381,6 +448,3 @@ gtk_toggle_tool_button_get_active (GtkToggleToolButton *button)
 
   return button->priv->active;
 }
-
-#define __GTK_TOGGLE_TOOL_BUTTON_C__
-#include "gtkaliasdef.c"
